(* Program to convert drbar masses of stops and gluinos to pole masses. 
 *
 * Uses TSIL to evaluate self-energies
 *
 * relevant papers:
 *
 *   [EFFPOT] S.P. Martin, " Two loop effective potential for the 
 *     minimal supersymmetric standard model", hep-ph/0206136
 *   [TSIL] S.P. Martin, "TSIL: Aa program for the calculation of
 *     two-loop self-energy integrals", hep-ph/0501132
 *   [FERMION] S.P. Martin, "Fermion self-energies and pole masses
 *     at two-loop order in a general renormalisable theory with
 *     massless gauge bosons", hep-ph/0509115
 *   [SCALAR] S.P. Martin, "Two-loop scalar self-energies and pole
 *     masses in a general renormalisable theory with massless 
 *     gauge bosons", hep-ph/0502168
 *   [SCALAR'] S.P. Martin, " Two loop scalar self energies in a 
 *     general renormalizable theory at leading order in gauge 
 *     couplings", hep-ph/0312092
 *)

(* {{{ set up all relevant constants *)

(* {{{ paths to the TSIL programs that evaluate the integrals *)
$EvalBCmd = "~/q2eexpcalc3/calc3/problems/c_mh/math/tsil/evalB";
(* }}} *)
(* {{{ lagrangian parameters *)

(* === squark mixing === *)
(* entries of mixing matrices, see Eq. (2.41) of [EFFPOT] 
 * we assume a real matrix, and thus use a mixing angle 
 *)
L[q_, 1] := Cos[theta[q]];
L[q_, 2] := - Sin[theta[q]];
R[q_, 1] := Sin[theta[q]];
R[q_, 2] := Cos[theta[q]];
(* mixing angles *)
theta[u]=0;
theta[d]=0;
theta[s]=0;
theta[c]=0;
(* theta[t]=0;                                                       *)
theta[b]=0;
(* shorthand notation defined in [SCALAR], eqs. (4.5, 4.6) *)
PP[q_, i_, j_] := L[q, i] L[q, j] - R[q, i] R[q, j];
NN[q_, i_, j_] := L[q, i] R[q, j] + R[q, i] L[q, j];
(* === quark and squark squared masses === *)
Do[ quark[q] = 0;
    squark[q,1]=Msq^2;
    squark[q,2]=Msq^2;
    ,
    {q, {u,d,s,c,b} } ];

quark[t]=0;
squark[t,1]=Mst^2;
squark[t,2]=Mst^2;
(* }}} *)

(* tr=1/2;                                                           *)
(* ca=3;                                                             *)


(* }}} *)


(* {{{ functions *)
(* function to convert a real number to a string that can be passed 
   to a c program *)
realToString[x_]:=ToString[NumberForm[x, NumberFormat->(SequenceForm[#1, "E", #3] &)]]
(* Kronecker delta *)
delta[i_?IntegerQ, j_?IntegerQ] := If[ i == j, 1, 0 ];
(* === functions in S.P.Martin's notation === 
 * the parameters s and qq, which are suppressed in the papers
 * [TSIL, FERMION, SCALAR], are restored and appear as the last
 * two arguments in the following functions
 *)

(* === integral functions === *)
ln[x_?NumberQ, y_?NumberQ] := Log[x/y];
TsilA[x_, qq_] := x(ln[x,qq]-1);

TsilB[x_?NumberQ, y_?NumberQ, s_?NumberQ, qq_?NumberQ] := TsilB[x, y, s, qq] =
    RunThrough[($EvalBCmd <> " "
                <>realToString[x] <> " "
                <>realToString[y] <> " "
                <>realToString[s] <> " "
                <>realToString[qq]), Null ];
               
(* Eqs. (3.3, 3.4) of [FERMION] *)
BFS[x_, y_, s_, qq_]       := ((y - x - s) TsilB[x, y, s, qq] - TsilA[x, qq] + TsilA[y, qq])/2;
BFbarS[x_, y_, s_, qq_]    := - TsilB[x, y, s, qq];
(* Eqs. (3.4 - 3.6) of [SCALAR] *)
BSS[x_, y_, s_, qq_]       := - TsilB[x, y, s, qq];
BFbarFbar[x_, y_, s_, qq_] := 2 TsilB[x, y, s, qq];
BFF[x_, y_, s_, qq_]       := (x + y - s) TsilB[x, y, s, qq] - TsilA[x, qq] - TsilA[y, qq];
BSV[x_, 0, s_, qq_]        := ((3 - xi) (x + s) TsilB[0, x, s, qq]
                               + (3 - 2 xi) TsilA[x, qq]
                               + 2 (xi - 1) s );

(* Eqs. (4.3, 4.11) of [SCALAR'] *)
XSSS[x_, y_, z_, qq_] := If[ 
    x == y,
    TsilA[z, qq] ln[x, qq],
    TsilA[z, qq] (TsilA[x, qq] - TsilA[y, qq]) / (x-y)];

(* Eqs. (4.16, 4.17) of [SCALAR'] *)
WSSFF[x_, y_, z_, u_, qq_] := If[
    x == y,
    ( (z + u - x) TsilI2p[x, z, u, qq] - TsilI2[x, z, u, qq]
     -( TsilA[u, qq] + TsilA[z, qq] ) ln[x, qq]
     ),
    ( ( (z + u - x) TsilI2[x, z, u, qq] 
       - TsilA[x, qq]( TsilA[z, qq] + TsilA[u, qq] ) )/(x - y) 
     +( (z + u - y) TsilI2[y, z, u, qq] 
       - TsilA[y, qq]( TsilA[z, qq] + TsilA[u, qq] ) )/(y - x) 
     )];
(* Eqs. (4.2, 4.10, 4.15) of [SCALAR'] *)
WSSSS[x_, y_, z_, u_, qq_] := If[
    x == y,
    ( - TsilI2p[x, z, u, qq] ),
    ( (TsilI2[x, z, u, qq] - TsilI2[y, z, u, qq])/(y - x) )];
WSSFbarFbar[x_, y_, z_, u_, qq_] := - 2 WSSSS[x, y, z, u, qq];
(* Vbar from Appendix A, [SCALAR'], Eqs. (A.11 - A.13) *)
Vbar[x_, 0, z_, u_, s_, qq_] := Which[
    ( (z == 0) && (u == 0) ),
    ( ( -(s+x) TsilTbar[0,0,x,s,qq] + 2 TsilS[0,0,x,s,qq]
       -(s+x) TsilB[0,x,s,qq] -3 TsilA[x,qq] 
       +x-s/4 )/(s-x)^2 ),
    ( u == z ),
    (
     1/(3 z (s-x)^2) 
     ( (4 z-3 x-s) z TsilT[z,z,x,s,qq]
      +(s-x) x TsilT[x,z,z,s,qq] 
      + (4 z-x+s) TsilS[x,z,z,s,qq]
      -2 z TsilI2[0,z,z,qq]
      + 2 TsilA[z,qq] ( x-2 z-TsilA[x,qq])
      + TsilA[x,qq] ( x/2-4 z-s/2 )
      + 4 z^2-z x - x^2 
      + 13 s x/8 + s z/2 - 5 s^2/8
      )
     - TsilB[0,x,s,qq] ( ( TsilA[z,qq]/z+1) (s+x)/(s-x)^2 + 1/6 z )
     ),
    True,
    (
     -x TsilU[x,0,z,u,s,qq]/(s-x)^2 
     +( (z+u)/((z-u)^2(s-x)) - 1/((s-x)^2) ) x TsilT[x,z,u,s,qq]
     +( (u(z-u-x+s))/((z-u)^3 (s-x)) -(2 x)/((z-u)(s-x)^2)
       ) z TsilT[z,x,u,s,qq]
     
     +( (z(z-u+x-s))/((z-u)^3 (s-x)) +(2 x)/((z-u)(s-x)^2) 
       ) u TsilT[u,x,z,s,qq]
     + 
     (
      (z+u) ( S(x,z,u) -TsilA[x,qq]/2 +x+z+u-5 s/8 ) + TsilA[z,qq] TsilA[u,qq] 
      )/((z-u)^2(s-x))
     + TsilB[0,x,s,qq] (((2 x+s) ( TsilA[u,qq]-TsilA[z,qq] ))/((s-x)^2 (z-u)) 
                        + (u TsilA[z,qq] - z TsilA[u,qq])/((z-u)^3)
                        - ((z+u))/(2(z-u)^2) )
     + (TsilA[x,qq] ( TsilA[u,qq] - TsilA[z,qq] ))/((z-u)(s-x)^2)
     + 
     (
      (x z+x u-2 s u) TsilA[z,qq] 
      + (x z+x u-2 s z) TsilA[u,qq] 
      )/[(z-u)^2(s-x)^2]
     )];
(* Eqs. (4.44, 4.43, 4.38) of [SCALAR'] *)
VFFFFS[x_, y_, z_, u_, v_, s_, qq_] := Which[
    ( (y == 0) && (z == 0) ),
    ((
      (u-v)(x-s) Vbar[x,0,u,v,s,qq]
      + (s - x - u +v) TsilU[x,0,u,v,s,qq]
      +TsilS[x,u,v,s,qq]
      +TsilI2[0,u,v,qq] )/2
     + (TsilA[u,qq] - TsilA[v,qq])(s TsilBp[x,0,s,qq] + TsilA[x,qq]/x)
     ),
    ( z == y ),
    (
     (
      (y+u-v) ((x+y-s) TsilV[x,y,u,v,s,qq] + TsilI2p[y,u,v,qq])
      + (s - x - 2 y - u +v) TsilU[x,y,u,v,s,qq]
      +TsilS[x,u,v,s,qq]
      +TsilI2[y,u,v,qq]
      + (TsilA[u,qq] - TsilA[v,qq])((s-x-y)TsilBp[y,x,s,qq] -TsilB[x,y,s,qq]
                                    + ln[y,qq])
      )/2.
     ),
    True,
    (
     (
     (y+u - v) ((s-x -y) U(x, y, u, v)+ I(y,u, v)) + y S(x,u,v)
     + (A(u)-A(v))( (s-x-y) B(x,y) + A(y) )
     )/2(y-z)
     +
     (
      (x+u - v) ((s-y -x) U(y, x, u, v)+ I(x,u, v)) + x S(y,u,v)
      + (A(u)-A(v))( (s-y-x) B(y,x) + A(x) )
      )/2(x-z)
     )
    ];



(* }}} *)



(* {{{ self-energies, evaluated on-shell *)

(* === one-loop gluino self-energy, Eq. (5.2) of [FERMION] === *)
piTildeGl1[mu_] := ca mgl^2 (10 - 6 ln[mgl^2,mu^2]) + 
    Sum[ 4 tr ( BFS[quark[q], squark[q,j], mgl^2, mu^2] 
               - 2 L[q,j] R[q,j] Sqrt[quark[q]] mgl BFbarS[q, sq[q,j], mgl^2, mu^2]  ),
         {q, {u,d,s,c,t,b}},
         {j, {1,2}}];

(* === one-loop squark self-energy, Eq. (4.7) of [SCALAR] === *)
(* the factor of g_3^2 is moved to the coefficient, in order to
 * use a uniform notation for squarks and gluinos
 *)
piTildeSq1[q_, i_, j_, mu_] := cf Sum[
    PP[q, i, k] PP[q, k, j] TsilA[squark[q,k], mu^2]
    + 2 delta[i, j] BFF[quark[q], mgl^2, squark[q, i], mu^2]
    - 2 NN[q, i, j] Sqrt[quark[q]] mgl BFbarFbar[quark[q], mgl^2, squark[q, i], mu^2]
    + delta[i, j] BSV[squark[q, i], 0, squark[q, i], mu^2]
    , {k, {1,2}}];
(* === two-loop squark self-energy, [SCALAR] Eqs. (4.7 - 4.10, 4.12) === *)
(* as above, the factor of g_3^4 will be absorbed in the coefficient *)
piSq20[q_, i_, j_, s_, mu_] := 

(* ---------------------------------------------------------------------- *)
\Pi^{(2,0)j}_i &=& 
4  cf^2 (
((1)/(4)) PP[q, i, k] PP[q, k, m] PP[q, m, n] PP[q, n, j] 
         XSSS[squark[q, ki], squark[q, ni],squark[q, mi], mu^2]
+  ((1)/(2)) PP[q, i, k] PP[q, k, j] 
         WSSFF[squark[q, ki],squark[q, ki],quark[q],mgl^2, mu^2]
(* \nonumber \\ &&                                                   *)
-  ((1)/(2)) PP[q, i, k] NN[q, k, m] PP[q, m, j] 
   Sqrt[quark[q]] mgl
         WSSFbarFbar[squark[q, ki],squark[q, mi],quark[q],mgl^2, mu^2]
(* ---------------------------------------------------------------------- *)
(* stopped here                                                      *)
(* ---------------------------------------------------------------------- *)
+  
  (L[q, i] L[q, j] |L[q, k]|^2  
  +R[q, i] R[q, j] |R[q, k]|^2)
 \propV_{FFFFS} (mgl^2,quark[q],quark[q],mgl^2,squark[q, ki])
(* \nonumber \\ &&                                                   *)
- \delta_i^j NN[q, k, k]
  Sqrt[quark[q]] mgl 
  \propV_{FF\Fbar\Fbar S}
  (mgl^2,q,q,mgl^2,squark[q, ki])
+  
  (L[q, i] R[q, j] R[q, k] L[q, k]
  +R[q, i] L[q, j] L[q, k] R[q, k])
  mgl^2
  \propV_{\Fbar FF\Fbar S}
  (mgl^2,q,q,mgl^2,squark[q, ki])
(* \nonumber \\ &&                                                   *)
+ 
  (L[q, i] L[q, j] |R[q, k]|^2
  +R[q, i] R[q, j] |L[q, k]|^2)
  m_q^2
  \propV_{F\Fbar\Fbar FS}
  (mgl^2,q,q,mgl^2,squark[q, ki])
-NN[q, i, j] 
  m_q mgl
  \propV_{\Fbar F\Fbar FS}
  (mgl^2,q,q,mgl^2,squark[q, ki])
(* \nonumber \\ &&                                                   *)
+
  (L[q, i] R[q, j] L[q, k] R[q, k]
  +R[q, i] L[q, j] R[q, k] L[q, k])
  m_q^2 mgl^2
  \propV_{\Fbar\Fbar\Fbar\Fbar S}
  (mgl^2,q,q,mgl^2,squark[q, ki])
)
(* \nonumber \\ &&                                                   *)
+ g_3^4 [4 C_q^2 - 2C_G C_q] \Bigl [
 (L[q, i] R[q, j] R[q, k] L[q, k]
 +R[q, i] L[q, j] L[q, k] R[q, k])
 \propM_{FFFFS} (quark[q],mgl^2,mgl^2,quark[q],squark[q, ki])
(* \nonumber \\ &&                                                   *)
- NN[q, i, j]
  Sqrt[quark[q]] mgl 
  \propM_{FF\Fbar\Fbar S}
  (quark[q],mgl^2,mgl^2,quark[q],squark[q, ki])
+ 
  (L[q, i] L[q, j] |L[q, k]|^2  
  +R[q, i] R[q, j] |R[q, k]|^2)
  mgl^2
  \propM_{F\Fbar\Fbar F S}
  (quark[q],mgl^2,mgl^2,quark[q],squark[q, ki])
(* \nonumber \\ &&                                                   *)
- \delta_i^j NN[q, k, k] 
  Sqrt[quark[q]] mgl
  \propM_{F\Fbar F\Fbar S}
  (quark[q],mgl^2,mgl^2,quark[q],squark[q, ki])
+ 
  (L[q, i] L[q, j] |R[q, k]|^2  
  +R[q, i] R[q, j] |L[q, k]|^2)
  Sqrt[quark[q]]^2
  \propM_{F\Fbar\Fbar F S}
  (mgl^2,quark[q],quark[q],mgl^2,squark[q, ki])
(* \nonumber \\ &&                                                   *)
+ 
  (L[q, i] R[q, j] L[q, k] R[q, k]  
  +R[q, i] L[q, j] R[q, k] L[q, k])  
  Sqrt[quark[q]]^2 mgl^2
  \propM_{\Fbar\Fbar\Fbar\Fbar S}
  (quark[q],mgl^2,mgl^2,quark[q],squark[q, ki])
+ ((1)/(4)) PP[q, i, k] PP[q, k, m] PP[q, m, n] PP[q, n, j]
   \propS_{SSS} (squark[q, ki], squark[q, mi],squark[q, ni])
\Bigr ]
(* \phantom{xxx}                                                     *)
(* \nonumber \\ &&                                                   *)
+ 4 g_3^4 C_q I_q \Bigl [
\delta_i^j   
 \propV_{FFFFS} (quark[q],mgl^2,mgl^2,q_r,squark[r, ri])
(* %\nonumber \\ &&                                                  *)
+ \delta_i^j  
  mgl^2
  \propV_{F\Fbar\Fbar F S}
  (q,mgl^2,mgl^2,q_r,squark[r, ri])
(* %\nonumber \\ &&                                                  *)
- 2 NN[q, i, j] 
  m_{q_r} mgl
  \propV_{\Fbar F\Fbar FS}
  (q,mgl^2,mgl^2,q_r,squark[r, ri])
(* \nonumber \\ &&                                                   *)
+ NN[q, i, j] NN[q, r, r] 
  m_q m_{q_r}
  \propV_{\Fbar FF\Fbar S}
  (q,mgl^2,mgl^2,q_r,squark[r, ri])
- 2 \delta_i^j NN[q, r, r] 
  m_{q_r} mgl
  \propV_{FF\Fbar\Fbar S}
  (q,mgl^2,mgl^2,q_r,squark[r, ri])
(* \nonumber \\ &&                                                   *)
+ NN[q, i, j] NN[q, r, r] 
  m_q m_{q_r} mgl^2
  \propV_{\Fbar\Fbar\Fbar\Fbar S}
  (q,mgl^2,mgl^2,q_r,squark[r, ri])
+ ((1)/(4)) PP[q, i, k] PP[q, k, j] PP[q, r, s] PP[q, s, r] 
   \propS_{SSS} (squark[q, ki], squark[r, ri],squark[s, si])
\Bigr ] .
(* ---------------------------------------------------------------------- *)
\Pi^{(2,1)j}_{i} &=& 
g_3^4 \delta_i^j \bigl (
C_q C_G [ G_{FF}(quark[q], mgl^2) 
+ G_{SSFF} (squark[q, i],squark[q, i],quark[q], mgl^2) ]
+ [2 C_q - C_G] C_q 
G_{SSFF} (squark[q, i],squark[q, i],mgl^2, quark[q])  \bigr )
(* \nonumber \\ &&                                                   *)
- g_3^4 
NN[q, i, j]
Sqrt[quark[q]] mgl
\bigl (
C_q C_G [ G_{\Fbar\Fbar}(quark[q], mgl^2)
+ G_{SS\Fbar\Fbar} (squark[q, i],squark[q, j],quark[q], mgl^2) ]
(* %\nonumber \\ &&                                                  *)
+ [2 C_q - C_G] C_q 
G_{SS\Fbar\Fbar} 
(squark[q, i],squark[q, j],mgl^2, quark[q])  \bigr )
(* \nonumber \\ &&                                                   *)
+ g_3^4 C_q^2 PP[q, i, k] PP[q, k, j] 
\left [
G_S(squark[q, ki]) + G_{SSS}(squark[q, i],squark[q, j],squark[q, ki])
\right ].
(* ---------------------------------------------------------------------- *)
\Pi^{(2,2)j}_i = g_3^4 \delta_i^j 
C_q \Bigl [
C_q F_1 (squark[q, i]) 
+ C_G F_2 (squark[q, i]) 
+ C_G F_3(squark[q, i], mgl^2)
+ I_q \sum_r ( F_3(squark[q, i],q_r)
+ F_4(squark[q, i],squark[r, ri]) )  
\Bigr ] .
(* ---------------------------------------------------------------------- *)







(* piTildeSq20[q_, i_, mu_] :=                                       *)
(* }}} *)
